////////////////////////////////////////////////////////////////////////////////
///                                                                          ///
/// AliFemtoEventReaderESDChain - the reader class for the Alice ESD         ///
/// tailored for the Task framework                                 ///
/// Reads in AliESDfriend to create shared hit/quality information           ///
/// Authors: Adam Kisiel kisiel@mps.ohio-state.edu                           ///
///                                                                          ///
////////////////////////////////////////////////////////////////////////////////
#include "AliFemtoEventReaderESDChain.h"

#include "TFile.h"
#include "TTree.h"
#include "AliESDEvent.h"
#include "AliESDtrack.h"
#include "AliESDVertex.h"
#include "AliMultiplicity.h"
#include "AliCentrality.h"
#include "AliEventplane.h"
#include "AliESDVZERO.h"
#include "AliFmPhysicalHelixD.h"
#include "AliFmThreeVectorF.h"
#include "SystemOfUnits.h"
#include "AliFemtoEvent.h"
#include "AliFemtoModelHiddenInfo.h"
#include "AliPID.h"

ClassImp(AliFemtoEventReaderESDChain)

#if !(ST_NO_NAMESPACES)
  using namespace units;
#endif

using namespace std;

//____________________________
AliFemtoEventReaderESDChain::AliFemtoEventReaderESDChain():
  fFileName(" "),
  fConstrained(true),
  fReadInner(false),
  fUseTPCOnly(false),
  fNumberofEvent(0),
  fCurEvent(0),
  fCurFile(0),
  fEvent(0x0),
  fUsePhysicsSel(kFALSE),
  fSelect(0x0),
  fTrackType(kGlobal),
  fEstEventMult(kITSTPC), 
  fEventTrig(AliVEvent::kMB), //trigger
  fESDpid(0),
  fIsPidOwner(0),
  fReadV0(0)
{
  //constructor with 0 parameters , look at default settings 
  //   fClusterPerPadrow = (list<Int_t> **) malloc(sizeof(list<Int_t> *) * AliESDfriendTrack::kMaxTPCcluster);
  //   for (int tPad=0; tPad<AliESDfriendTrack::kMaxTPCcluster; tPad++) {
  //     fClusterPerPadrow[tPad] = new list<Int_t>();
  //   }
  //   fSharedList = (list<Int_t> **) malloc(sizeof(list<Int_t> *) * AliESDfriendTrack::kMaxTPCcluster);
  //   for (int tPad=0; tPad<AliESDfriendTrack::kMaxTPCcluster; tPad++) {
  //     fSharedList[tPad] = new list<Int_t>();
  //   }
}

//__________________
AliFemtoEventReaderESDChain::AliFemtoEventReaderESDChain(const AliFemtoEventReaderESDChain& aReader):
  AliFemtoEventReader(aReader),
  fFileName(" "),
  fConstrained(true),
  fReadInner(false),
  fUseTPCOnly(false),
  fNumberofEvent(0),
  fCurEvent(0),
  fCurFile(0),
  fEvent(0x0),
  fUsePhysicsSel(kFALSE),
  fSelect(0x0),
  fTrackType(kGlobal),
  fEstEventMult(kITSTPC),
  fEventTrig(AliVEvent::kMB), //trigger
  fESDpid(0),
  fIsPidOwner(0),
  fReadV0(0)
{
  // Copy constructor
  fConstrained = aReader.fConstrained;
  fReadInner = aReader.fReadInner;
  fUseTPCOnly = aReader.fUseTPCOnly;
  fNumberofEvent = aReader.fNumberofEvent;
  fCurEvent = aReader.fCurEvent;
  fCurFile = aReader.fCurFile;
  //  fEvent = new AliESD(*aReader.fEvent);
  fEvent = new AliESDEvent();
  fUsePhysicsSel = aReader.fUsePhysicsSel;
  if (aReader.fUsePhysicsSel)
    fSelect = new AliPhysicsSelection();
  fTrackType = aReader.fTrackType;
  fEstEventMult = aReader.fEstEventMult;
  fEventTrig = aReader.fEventTrig; //trigger

  //   fEventFriend = aReader.fEventFriend;
  //   fClusterPerPadrow = (list<Int_t> **) malloc(sizeof(list<Int_t> *) * AliESDfriendTrack::kMaxTPCcluster);
  //   for (int tPad=0; tPad<AliESDfriendTrack::kMaxTPCcluster; tPad++) {
  //     fClusterPerPadrow[tPad] = new list<Int_t>();
  //     list<Int_t>::iterator iter;
  //     for (iter=aReader.fClusterPerPadrow[tPad]->begin(); iter!=aReader.fClusterPerPadrow[tPad]->end(); iter++) {
  //       fClusterPerPadrow[tPad]->push_back(*iter);
  //     }
  //   }
  //   fSharedList = (list<Int_t> **) malloc(sizeof(list<Int_t> *) * AliESDfriendTrack::kMaxTPCcluster);
  //   for (int tPad=0; tPad<AliESDfriendTrack::kMaxTPCcluster; tPad++) {
  //     fSharedList[tPad] = new list<Int_t>();
  //     list<Int_t>::iterator iter;
  //     for (iter=aReader.fSharedList[tPad]->begin(); iter!=aReader.fSharedList[tPad]->end(); iter++) {
  //       fSharedList[tPad]->push_back(*iter);
  //     }
  //   }
}
//__________________
AliFemtoEventReaderESDChain::~AliFemtoEventReaderESDChain()
{
  //Destructor
  delete fEvent;

  //   for (int tPad=0; tPad<AliESDfriendTrack::kMaxTPCcluster; tPad++) {
  //     fClusterPerPadrow[tPad]->clear();
  //     delete fClusterPerPadrow[tPad];
  //   }
  //   delete [] fClusterPerPadrow;
  //   for (int tPad=0; tPad<AliESDfriendTrack::kMaxTPCcluster; tPad++) {
  //     fSharedList[tPad]->clear();
  //     delete fSharedList[tPad];
  //   }
  //   delete [] fSharedList;
  if (fSelect) delete fSelect;
}

//__________________
AliFemtoEventReaderESDChain& AliFemtoEventReaderESDChain::operator=(const AliFemtoEventReaderESDChain& aReader)
{
  // Assignment operator
  if (this == &aReader)
    return *this;

  fConstrained = aReader.fConstrained;
  fReadInner = aReader.fReadInner;
  fUseTPCOnly = aReader.fUseTPCOnly;
  fNumberofEvent = aReader.fNumberofEvent;
  fCurEvent = aReader.fCurEvent;
  fCurFile = aReader.fCurFile;
  if (fEvent) delete fEvent;
  fEvent = new AliESDEvent();
  fTrackType = aReader.fTrackType;
  fEstEventMult = aReader.fEstEventMult;

  fUsePhysicsSel = aReader.fUsePhysicsSel;
  fReadV0 = aReader.fReadV0;
  if (aReader.fUsePhysicsSel)
    fSelect = new AliPhysicsSelection();
  //  fEventFriend = aReader.fEventFriend;
  
  //   if (fClusterPerPadrow) {
  //     for (int tPad=0; tPad<AliESDfriendTrack::kMaxTPCcluster; tPad++) {
  //       fClusterPerPadrow[tPad]->clear();
  //       delete fClusterPerPadrow[tPad];
  //     }
  //     delete [] fClusterPerPadrow;
  //   }
  
  //   if (fSharedList) {
  //     for (int tPad=0; tPad<AliESDfriendTrack::kMaxTPCcluster; tPad++) {
  //       fSharedList[tPad]->clear();
  //       delete fSharedList[tPad];
  //     }
  //     delete [] fSharedList;
  //   }

  //   fClusterPerPadrow = (list<Int_t> **) malloc(sizeof(list<Int_t> *) * AliESDfriendTrack::kMaxTPCcluster);
  //   for (int tPad=0; tPad<AliESDfriendTrack::kMaxTPCcluster; tPad++) {
  //     fClusterPerPadrow[tPad] = new list<Int_t>();
  //     list<Int_t>::iterator iter;
  //     for (iter=aReader.fClusterPerPadrow[tPad]->begin(); iter!=aReader.fClusterPerPadrow[tPad]->end(); iter++) {
  //       fClusterPerPadrow[tPad]->push_back(*iter);
  //     }
  //   }
  //   fSharedList = (list<Int_t> **) malloc(sizeof(list<Int_t> *) * AliESDfriendTrack::kMaxTPCcluster);
  //   for (int tPad=0; tPad<AliESDfriendTrack::kMaxTPCcluster; tPad++) {
  //     fSharedList[tPad] = new list<Int_t>();
  //     list<Int_t>::iterator iter;
  //     for (iter=aReader.fSharedList[tPad]->begin(); iter!=aReader.fSharedList[tPad]->end(); iter++) {
  //       fSharedList[tPad]->push_back(*iter);
  //     }
  //   }
  
  return *this;
}
//__________________
// Simple report
AliFemtoString AliFemtoEventReaderESDChain::Report()
{
  AliFemtoString temp = "\n This is the AliFemtoEventReaderESDChain\n";
  return temp;
}

//__________________
void AliFemtoEventReaderESDChain::SetConstrained(const bool constrained)
{
  // Select whether to read constrained or not constrained momentum
  fConstrained=constrained;
}
//__________________
bool AliFemtoEventReaderESDChain::GetConstrained() const
{
  // Check whether we read constrained or not constrained momentum
  return fConstrained;
}
//__________________
void AliFemtoEventReaderESDChain::SetReadTPCInner(const bool readinner)
{
  fReadInner=readinner;
}

bool AliFemtoEventReaderESDChain::GetReadTPCInner() const
{
  return fReadInner;
}

//__________________
void AliFemtoEventReaderESDChain::SetUseTPCOnly(const bool usetpconly)
{
  fUseTPCOnly=usetpconly;
}

bool AliFemtoEventReaderESDChain::GetUseTPCOnly() const
{
  return fUseTPCOnly;
}

void AliFemtoEventReaderESDChain::SetUsePhysicsSelection(const bool usephysics)
{
  fUsePhysicsSel = usephysics;
  if (!fSelect) fSelect = new AliPhysicsSelection();
}

void AliFemtoEventReaderESDChain::SetUseMultiplicity(EstEventMult aType)
{
  fEstEventMult = aType;
}

AliFemtoEvent* AliFemtoEventReaderESDChain::ReturnHbtEvent()
{
  // Get the event, read all the relevant information
  // and fill the AliFemtoEvent class
  // Returns a valid AliFemtoEvent
  AliFemtoEvent *hbtEvent = 0;
  string tFriendFileName;

  // Get the friend information
  if (Debug()>1) cout<<"starting to read event "<<fCurEvent<<endl;
  //  fEvent->SetESDfriend(fEventFriend);
  if(fEvent->GetAliESDOld())fEvent->CopyFromOldESD();
  
  hbtEvent = new AliFemtoEvent;

  if (fUsePhysicsSel) {
    hbtEvent->SetIsCollisionCandidate(fSelect->IsCollisionCandidate(fEvent));
    if (!(fSelect->IsCollisionCandidate(fEvent)))
      printf("Event not a collision candidate\n");
  }
  else
    hbtEvent->SetIsCollisionCandidate(kTRUE);

  //setting basic things
  //  hbtEvent->SetEventNumber(fEvent->GetEventNumber());
  hbtEvent->SetRunNumber(fEvent->GetRunNumber());
  //hbtEvent->SetNumberOfTracks(fEvent->GetNumberOfTracks());
  hbtEvent->SetMagneticField(fEvent->GetMagneticField()*kilogauss);//to check if here is ok
  hbtEvent->SetZDCN1Energy(fEvent->GetZDCN1Energy());
  hbtEvent->SetZDCP1Energy(fEvent->GetZDCP1Energy());
  hbtEvent->SetZDCN2Energy(fEvent->GetZDCN2Energy());
  hbtEvent->SetZDCP2Energy(fEvent->GetZDCP2Energy());
  hbtEvent->SetZDCEMEnergy(fEvent->GetZDCEMEnergy());
  hbtEvent->SetZDCParticipants(fEvent->GetZDCParticipants());
  hbtEvent->SetTriggerMask(fEvent->GetTriggerMask());
  //  hbtEvent->SetTriggerCluster(fEvent->GetTriggerCluster());

  if ((fEvent->IsTriggerClassFired("CINT1WU-B-NOPF-ALL")) ||
      (fEvent->IsTriggerClassFired("CINT1B-ABCE-NOPF-ALL")) ||
      (fEvent->IsTriggerClassFired("CINT1-B-NOPF-ALLNOTRD")) ||
      (fEvent->IsTriggerClassFired("CINT1-B-NOPF-FASTNOTRD")))
    hbtEvent->SetTriggerCluster(1);
  else if ((fEvent->IsTriggerClassFired("CSH1WU-B-NOPF-ALL")) ||
	   (fEvent->IsTriggerClassFired("CSH1-B-NOPF-ALLNOTRD")))
    hbtEvent->SetTriggerCluster(2);
  else 
    hbtEvent->SetTriggerCluster(0);
	
  //Vertex
  double fV1[3];
  double fVCov[6];
  if (fUseTPCOnly) {
    fEvent->GetPrimaryVertexTPC()->GetXYZ(fV1);
    fEvent->GetPrimaryVertexTPC()->GetCovMatrix(fVCov);
    if (!fEvent->GetPrimaryVertexTPC()->GetStatus())
      fVCov[4] = -1001.0;
  }
  else {
    fEvent->GetPrimaryVertex()->GetXYZ(fV1);
    fEvent->GetPrimaryVertex()->GetCovMatrix(fVCov);
    if (!fEvent->GetPrimaryVertex()->GetStatus())
      fVCov[4] = -1001.0;
  }

  AliFmThreeVectorF vertex(fV1[0],fV1[1],fV1[2]);
  hbtEvent->SetPrimVertPos(vertex);
  hbtEvent->SetPrimVertCov(fVCov);
  
  Int_t spdetaonecount = 0;
  
  for (int iter=0; iter<fEvent->GetMultiplicity()->GetNumberOfTracklets(); iter++) 
    if (fabs(fEvent->GetMultiplicity()->GetEta(iter)) < 1.0)
      spdetaonecount++;

  //  hbtEvent->SetSPDMult(fEvent->GetMultiplicity()->GetNumberOfTracklets());
  hbtEvent->SetSPDMult(spdetaonecount);

  //starting to reading tracks
  int nofTracks=0;  //number of reconstructed tracks in event
  nofTracks=fEvent->GetNumberOfTracks();
  int realnofTracks=0;//number of track which we use ina analysis

  //   // Clear the shared cluster list
  //   for (int tPad=0; tPad<AliESDfriendTrack::kMaxTPCcluster; tPad++) {
  //     fClusterPerPadrow[tPad]->clear();
  //   }
  //   for (int tPad=0; tPad<AliESDfriendTrack::kMaxTPCcluster; tPad++) {
  //     fSharedList[tPad]->clear();
  //   }


  //   for (int i=0;i<nofTracks;i++) {
  //     const AliESDtrack *esdtrack=fEvent->GetTrack(i);//getting next track

  //     list<Int_t>::iterator tClustIter;

  //     Int_t tTrackIndices[AliESDfriendTrack::kMaxTPCcluster];
  //     Int_t tNClusters = esdtrack->GetTPCclusters(tTrackIndices);
  //     for (int tNcl=0; tNcl<AliESDfriendTrack::kMaxTPCcluster; tNcl++) {
  //       if (tTrackIndices[tNcl] >= 0) {
  // 	tClustIter = find(fClusterPerPadrow[tNcl]->begin(), fClusterPerPadrow[tNcl]->end(), tTrackIndices[tNcl]);
  // 	if (tClustIter == fClusterPerPadrow[tNcl]->end()) {
  // 	  fClusterPerPadrow[tNcl]->push_back(tTrackIndices[tNcl]);
  // 	}
  // 	else {
  // 	  fSharedList[tNcl]->push_back(tTrackIndices[tNcl]);
  // 	}
  //       }
  //     }
      
  //   }

  int tNormMult = 0;
  int tNormMultPos = 0;
  int tNormMultNeg = 0;

  Float_t tTotalPt = 0.0;

  Float_t b[2];
  Float_t bCov[3];

  Int_t tTracklet=0, tITSTPC=0, tITSPure=0;
  
  fEvent->EstimateMultiplicity(tTracklet, tITSTPC, tITSPure, 1.2);
  
  hbtEvent->SetMultiplicityEstimateITSTPC(tITSTPC);
  hbtEvent->SetMultiplicityEstimateTracklets(tTracklet);
  //  hbtEvent->SetMultiplicityEstimateITSPure(tITSPure);
  hbtEvent->SetMultiplicityEstimateITSPure(fEvent->GetMultiplicity()->GetNumberOfITSClusters(1));
  
  for (int i=0;i<nofTracks;i++)
    {
      bool  tGoodMomentum=true; //flaga to chcek if we can read momentum of this track

      const AliESDtrack *esdtrack=fEvent->GetTrack(i);//getting next track
      //      const AliESDfriendTrack *tESDfriendTrack = esdtrack->GetFriendTrack();

      if ((esdtrack->GetStatus() & AliESDtrack::kTPCrefit) &&
	  (esdtrack->GetStatus() & AliESDtrack::kITSrefit)) {
	if (esdtrack->GetTPCNcls() > 70) 
	  if (esdtrack->GetTPCchi2()/esdtrack->GetTPCNcls() < 4.0) {
	    if (TMath::Abs(esdtrack->Eta()) < 1.2) {
	      esdtrack->GetImpactParameters(b,bCov);
	      if ((b[0]<0.2) && (b[1] < 0.25)) {
		tNormMult++;
		tTotalPt += esdtrack->Pt();
	      }
	    }
	  }
      }
      else if (esdtrack->GetStatus() & AliESDtrack::kTPCrefit) {
	if (esdtrack->GetTPCNcls() > 100) 
	  if (esdtrack->GetTPCchi2()/esdtrack->GetTPCNcls() < 4.0) {
	    if (TMath::Abs(esdtrack->Eta()) < 1.2) {
	      esdtrack->GetImpactParameters(b,bCov);
	      if ((b[0]<2.4) && (b[1] < 3.2)) {
		tNormMult++;
		tTotalPt += esdtrack->Pt();
	      }
	    }
	  }
      }
      
      hbtEvent->SetZDCEMEnergy(tTotalPt);
      //       if (esdtrack->GetStatus() & AliESDtrack::kTPCrefit)
      // 	if (esdtrack->GetTPCNcls() > 80) 
      // 	  if (esdtrack->GetTPCchi2()/esdtrack->GetTPCNcls() < 6.0) 
      // 	    if (esdtrack->GetConstrainedParam())
      // 	      if (fabs(esdtrack->GetConstrainedParam()->Eta()) < 0.5)
      // 		if (esdtrack->GetConstrainedParam()->Pt() < 1.0) {
      // 		  if (esdtrack->GetSign() > 0)
      // 		    tNormMultPos++;
      // 		  else if (esdtrack->GetSign() < 0)
      // 		    tNormMultNeg--;
      // 		}

      // If reading ITS-only tracks, reject all with TPC
      if (fTrackType == kITSOnly) {
	if (esdtrack->GetStatus() & AliESDtrack::kTPCrefit) continue;
	if (!(esdtrack->GetStatus() & AliESDtrack::kITSrefit)) continue;
	if (esdtrack->GetStatus() & AliESDtrack::kTPCin) continue;
	UChar_t iclm = esdtrack->GetITSClusterMap();
	Int_t incls = 0;
	for (int iter=0; iter<6; iter++) if (iclm&(1<<iter)) incls++;
	if (incls<=3) {
	  if (Debug()>1) cout << "Rejecting track with " << incls << " clusters" << endl;
	  continue;
	}
      }

      AliFemtoTrack* trackCopy = new AliFemtoTrack();	
      trackCopy->SetCharge((short)esdtrack->GetSign());

      //in aliroot we have AliPID 
      //0-electron 1-muon 2-pion 3-kaon 4-proton 5-photon 6-pi0 7-neutron 8-kaon0 9-eleCon   
      //we use only 5 first
      double esdpid[5];
      //       esdtrack->GetESDpid(esdpid);
      esdtrack->GetTPCpid(esdpid);
      trackCopy->SetPidProbElectron(esdpid[0]);
      trackCopy->SetPidProbMuon(esdpid[1]);
      trackCopy->SetPidProbPion(esdpid[2]);
      trackCopy->SetPidProbKaon(esdpid[3]);
      trackCopy->SetPidProbProton(esdpid[4]);
      
      esdpid[0] = -100000.0;
      esdpid[1] = -100000.0;
      esdpid[2] = -100000.0;
      esdpid[3] = -100000.0;
      esdpid[4] = -100000.0;
      
      double tTOF = 0.0;

      if (esdtrack->GetStatus()&AliESDtrack::kTOFpid) {
	tTOF = esdtrack->GetTOFsignal();
	esdtrack->GetIntegratedTimes(esdpid);
      }

      trackCopy->SetTofExpectedTimes(tTOF-esdpid[2], tTOF-esdpid[3], tTOF-esdpid[4]);

      //////  TPC ////////////////////////////////////////////

      float nsigmaTPCK=-1000.;                                                        
      float nsigmaTPCPi=-1000.;                                                        
      float nsigmaTPCP=-1000.;                                                        
          
  
      if ((fESDpid) && (esdtrack->IsOn(AliESDtrack::kTPCpid))){
        nsigmaTPCK = fESDpid->NumberOfSigmasTPC(esdtrack,AliPID::kKaon);
        nsigmaTPCPi = fESDpid->NumberOfSigmasTPC(esdtrack,AliPID::kPion);
        nsigmaTPCP = fESDpid->NumberOfSigmasTPC(esdtrack,AliPID::kProton);
	
      }
      trackCopy->SetNSigmaTPCPi(nsigmaTPCPi);
      trackCopy->SetNSigmaTPCK(nsigmaTPCK);
      trackCopy->SetNSigmaTPCP(nsigmaTPCP);
      
      ///// TOF ///////////////////////////////////////////////
	
	float vp=-1000.;
	float nsigmaTOFPi=-1000.;
	float nsigmaTOFK=-1000.;
	float nsigmaTOFP=-1000.;
	
	if ((esdtrack->GetStatus()&AliESDtrack::kTOFpid) &&
	    (esdtrack->GetStatus()&AliESDtrack::kTOFout) &&
	    (esdtrack->GetStatus()&AliESDtrack::kTIME) &&
	    !(esdtrack->GetStatus()&AliESDtrack::kTOFmismatch))
	  {

	    //if ((esdtrack->GetStatus()&AliESDtrack::kTOFpid) &&
	    //(esdtrack->GetStatus()&AliESDtrack::kTOFout) &&
	    //(esdtrack->GetStatus()&AliESDtrack::kTIME)){
	    // collect info from ESDpid class
	  
	    if ((fESDpid) && (esdtrack->IsOn(AliESDtrack::kTOFpid))) {
	      
	      
	      double tZero = fESDpid->GetTOFResponse().GetStartTime(esdtrack->P());
	      
	      nsigmaTOFPi = fESDpid->NumberOfSigmasTOF(esdtrack,AliPID::kPion,tZero);
	      nsigmaTOFK = fESDpid->NumberOfSigmasTOF(esdtrack,AliPID::kKaon,tZero);
	      nsigmaTOFP = fESDpid->NumberOfSigmasTOF(esdtrack,AliPID::kProton,tZero);
	      
	      Double_t len=esdtrack->GetIntegratedLength();
	      Double_t tof=esdtrack->GetTOFsignal();
	      if(tof > 0.) vp=len/tof/0.03;
	    }
	  }
	
	trackCopy->SetVTOF(vp);
	trackCopy->SetNSigmaTOFPi(nsigmaTOFPi);
	trackCopy->SetNSigmaTOFK(nsigmaTOFK);
	trackCopy->SetNSigmaTOFP(nsigmaTOFP);
	
	double pxyz[3];
	double rxyz[3];
	double impact[2];
	double covimpact[3];
      
	if (fUseTPCOnly) {
	  if (!esdtrack->GetTPCInnerParam()) {
	    delete trackCopy;
	    continue;
	  }


	  AliExternalTrackParam *param = new AliExternalTrackParam(*esdtrack->GetTPCInnerParam());
	  param->GetXYZ(rxyz);
	  param->PropagateToDCA(fEvent->GetPrimaryVertexTPC(), (fEvent->GetMagneticField()), 10000, impact, covimpact);
	  param->GetPxPyPz(pxyz);//reading noconstarined momentum

	  if (fReadInner == true) {
	    AliFemtoModelHiddenInfo *tInfo = new AliFemtoModelHiddenInfo();
	    tInfo->SetPDGPid(211);
	    tInfo->SetTrueMomentum(pxyz[0], pxyz[1], pxyz[2]);
	    tInfo->SetMass(0.13957);
	    //	  tInfo->SetEmissionPoint(rxyz[0], rxyz[1], rxyz[2], 0.0);
	    //	  tInfo->SetEmissionPoint(fV1[0], fV1[1], fV1[2], 0.0);
	    tInfo->SetEmissionPoint(rxyz[0]-fV1[0], rxyz[1]-fV1[1], rxyz[2]-fV1[2], 0.0);
	    trackCopy->SetHiddenInfo(tInfo);
	  }
	
	  AliFemtoThreeVector v(pxyz[0],pxyz[1],pxyz[2]);
	  if (v.Mag() < 0.0001) {
	    //	cout << "Found 0 momentum ???? " <<endl;
	    delete trackCopy;
	    continue;
	  }
	  trackCopy->SetP(v);//setting momentum
	  trackCopy->SetPt(sqrt(pxyz[0]*pxyz[0]+pxyz[1]*pxyz[1]));

	  const AliFmThreeVectorD kP(pxyz[0],pxyz[1],pxyz[2]);
	  const AliFmThreeVectorD kOrigin(fV1[0],fV1[1],fV1[2]);
	  //setting helix I do not if it is ok
	  AliFmPhysicalHelixD helix(kP,kOrigin,(double)(fEvent->GetMagneticField())*kilogauss,(double)(trackCopy->Charge())); 
	  trackCopy->SetHelix(helix);

	  //some stuff which could be useful 
	  trackCopy->SetImpactD(impact[0]);
	  trackCopy->SetImpactZ(impact[1]);
	  trackCopy->SetCdd(covimpact[0]);
	  trackCopy->SetCdz(covimpact[1]);
	  trackCopy->SetCzz(covimpact[2]);
	  trackCopy->SetSigmaToVertex(GetSigmaToVertex(impact, covimpact));	

	  delete param;
	}
	else {
	  if (fReadInner == true) {
	  
	    if (esdtrack->GetTPCInnerParam()) {
	      AliExternalTrackParam *param = new AliExternalTrackParam(*esdtrack->GetTPCInnerParam());
	      param->GetXYZ(rxyz);
	      //	    param->PropagateToDCA(fEvent->GetPrimaryVertex(), (fEvent->GetMagneticField()), 10000);
	      param->GetPxPyPz(pxyz);//reading noconstarined momentum
	      delete param;
	    
	      AliFemtoModelHiddenInfo *tInfo = new AliFemtoModelHiddenInfo();
	      tInfo->SetPDGPid(211);
	      tInfo->SetTrueMomentum(pxyz[0], pxyz[1], pxyz[2]);
	      tInfo->SetMass(0.13957);
	      //	    tInfo->SetEmissionPoint(rxyz[0], rxyz[1], rxyz[2], 0.0);
	      //tInfo->SetEmissionPoint(fV1[0], fV1[1], fV1[2], 0.0);
	      tInfo->SetEmissionPoint(rxyz[0]-fV1[0], rxyz[1]-fV1[1], rxyz[2]-fV1[2], 0.0);
	      trackCopy->SetHiddenInfo(tInfo);
	    }
	  }

	  if (fTrackType == kGlobal) {
	    if (fConstrained==true)		    
	      tGoodMomentum=esdtrack->GetConstrainedPxPyPz(pxyz); //reading constrained momentum
	    else
	      tGoodMomentum=esdtrack->GetPxPyPz(pxyz);//reading noconstarined momentum
	  }
	  else if (fTrackType == kTPCOnly) {
	    if (esdtrack->GetTPCInnerParam())
	      esdtrack->GetTPCInnerParam()->GetPxPyPz(pxyz);
	    else {
	      delete trackCopy;
	      continue;
	    }
	  }
	  else if (fTrackType == kITSOnly) {
	    if (fConstrained==true)		    
	      tGoodMomentum=esdtrack->GetConstrainedPxPyPz(pxyz); //reading constrained momentum
	    else
	      tGoodMomentum=esdtrack->GetPxPyPz(pxyz);//reading noconstarined momentum
	  }


	  AliFemtoThreeVector v(pxyz[0],pxyz[1],pxyz[2]);
	  if (v.Mag() < 0.0001) {
	    //	cout << "Found 0 momentum ???? " <<endl;
	    delete trackCopy;
	    continue;
	  }
	  trackCopy->SetP(v);//setting momentum
	  trackCopy->SetPt(sqrt(pxyz[0]*pxyz[0]+pxyz[1]*pxyz[1]));
	  const AliFmThreeVectorD kP(pxyz[0],pxyz[1],pxyz[2]);
	  const AliFmThreeVectorD kOrigin(fV1[0],fV1[1],fV1[2]);
	  //setting helix I do not if it is ok
	  AliFmPhysicalHelixD helix(kP,kOrigin,(double)(fEvent->GetMagneticField())*kilogauss,(double)(trackCopy->Charge())); 
	  trackCopy->SetHelix(helix);

	  //some stuff which could be useful 
	  float imp[2];
	  float cim[3];
	  // if (fTrackType == kTPCOnly) {
	  //   esdtrack->GetTPCInnerParam()->GetImpactParameters(imp,cim);
	  // }
	  // else {
	  esdtrack->GetImpactParameters(imp,cim);
	  // }

	  impact[0] = imp[0];
	  impact[1] = imp[1];
	  covimpact[0] = cim[0];
	  covimpact[1] = cim[1];
	  covimpact[2] = cim[2];

	  trackCopy->SetImpactD(impact[0]);
	  trackCopy->SetImpactZ(impact[1]);
	  trackCopy->SetCdd(covimpact[0]);
	  trackCopy->SetCdz(covimpact[1]);
	  trackCopy->SetCzz(covimpact[2]);
	  trackCopy->SetSigmaToVertex(GetSigmaToVertex(impact,covimpact));
	}

	trackCopy->SetTrackId(esdtrack->GetID());
	trackCopy->SetFlags(esdtrack->GetStatus());
	trackCopy->SetLabel(esdtrack->GetLabel());
		
	trackCopy->SetITSchi2(esdtrack->GetITSchi2());    
	if (esdtrack->GetITSFakeFlag())
	  trackCopy->SetITSncls(-esdtrack->GetNcls(0));     
	else
	  trackCopy->SetITSncls(esdtrack->GetNcls(0));     
	trackCopy->SetTPCchi2(esdtrack->GetTPCchi2());       
	trackCopy->SetTPCncls(esdtrack->GetTPCNcls());       
	trackCopy->SetTPCnclsF(esdtrack->GetTPCNclsF());      
	trackCopy->SetTPCsignal(esdtrack->GetTPCsignal());
	trackCopy->SetTPCsignalN((short)esdtrack->GetTPCsignalN()); //due to bug in aliesdtrack class   
	trackCopy->SetTPCsignalS(esdtrack->GetTPCsignalSigma()); 

	trackCopy->SetTPCClusterMap(esdtrack->GetTPCClusterMap());
	trackCopy->SetTPCSharedMap(esdtrack->GetTPCSharedMap());

	double xtpc[3];
	esdtrack->GetInnerXYZ(xtpc);
	xtpc[2] -= fV1[2];
	trackCopy->SetNominalTPCEntrancePoint(xtpc);

	esdtrack->GetOuterXYZ(xtpc);
	xtpc[2] -= fV1[2];
	trackCopy->SetNominalTPCExitPoint(xtpc);

	int indexes[3];
	for (int ik=0; ik<3; ik++) {
	  indexes[ik] = esdtrack->GetKinkIndex(ik);
	}
	trackCopy->SetKinkIndexes(indexes);
	//decision if we want this track
	//if we using diffrent labels we want that this label was use for first time 
	//if we use hidden info we want to have match between sim data and ESD
	if (tGoodMomentum==true)
	  {
	    hbtEvent->TrackCollection()->push_back(trackCopy);//adding track to analysis
	    realnofTracks++;//real number of tracks
	    //	  delete trackCopy;
	  }
	else
	  {
	    delete  trackCopy;
	  }
		
    }

  hbtEvent->SetNumberOfTracks(realnofTracks);//setting number of track which we read in event	

  AliCentrality *cent = fEvent->GetCentrality();
  if (cent) {
    hbtEvent->SetCentralityV0(cent->GetCentralityPercentile("V0M"));
    //    hbtEvent->SetCentralityFMD(cent->GetCentralityPercentile("FMD"));
    hbtEvent->SetCentralitySPD1(cent->GetCentralityPercentile("CL1"));
    //    hbtEvent->SetCentralityTrk(cent->GetCentralityPercentile("TRK"));

    if (Debug()>1) printf("  FemtoReader Got Event with %f %f %f %f\n", cent->GetCentralityPercentile("V0M"), 0.0, cent->GetCentralityPercentile("CL1"), 0.0);
  }

  if (fEstEventMult == kGlobalCount) 
    hbtEvent->SetNormalizedMult(tNormMult);
  else if (fEstEventMult == kTracklet)
    hbtEvent->SetNormalizedMult(tTracklet);
  else if (fEstEventMult == kITSTPC)
    hbtEvent->SetNormalizedMult(tITSTPC);
  else if (fEstEventMult == kITSPure)
    hbtEvent->SetNormalizedMult(tITSPure);
  else if (fEstEventMult == kSPDLayer1)
    hbtEvent->SetNormalizedMult(fEvent->GetMultiplicity()->GetNumberOfITSClusters(1));
  else if (fEstEventMult == kV0Centrality) {
    // centrality between 0 (central) and 1 (very peripheral)

    if (cent) {
      if (cent->GetCentralityPercentile("V0M") < 0.00001)
	hbtEvent->SetNormalizedMult(-1);
      else
	hbtEvent->SetNormalizedMult(lrint(10.0*cent->GetCentralityPercentile("V0M")));
      if (Debug()>1) printf ("Set Centrality %i %f %li\n", hbtEvent->UncorrectedNumberOfPrimaries(), 
			     10.0*cent->GetCentralityPercentile("V0M"), lrint(10.0*cent->GetCentralityPercentile("V0M")));
    }
  }

  if (tNormMultPos > tNormMultNeg)
    hbtEvent->SetZDCParticipants(tNormMultPos);
  else
    hbtEvent->SetZDCParticipants(tNormMultNeg);
  
  AliEventplane* ep = fEvent->GetEventplane();
  if (ep) {
    hbtEvent->SetEP(ep);
    hbtEvent->SetReactionPlaneAngle(ep->GetEventplane("Q"));
  }

  //V0
  if(fReadV0)
    {
      for (Int_t i = 0; i < fEvent->GetNumberOfV0s(); i++) {
	AliESDv0* esdv0 = fEvent->GetV0(i);
	if (!esdv0) continue;
	//if(esdv0->GetNDaughters()>2) continue;
	//if(esdv0->GetNProngs()>2) continue;
	if(esdv0->Charge()!=0) continue;
	AliESDtrack *trackPos = fEvent->GetTrack(esdv0->GetPindex());
	if(!trackPos) continue;
	AliESDtrack *trackNeg = fEvent->GetTrack(esdv0->GetNindex());
	if(!trackNeg) continue;
	if(trackPos->Charge()==trackNeg->Charge()) continue;
	AliFemtoV0* trackCopyV0 = new AliFemtoV0();
	CopyESDtoFemtoV0(esdv0, trackCopyV0, fEvent);
	hbtEvent->V0Collection()->push_back(trackCopyV0);
	//cout<<"Pushback v0 to v0collection"<<endl;
      }
    }


  fCurEvent++;	
  if (Debug()>1) cout<<"end of reading nt "<<nofTracks<<" real number "<<realnofTracks<<endl;

  return hbtEvent; 
}
//___________________
void AliFemtoEventReaderESDChain::SetESDSource(AliESDEvent *aESD)
{
  // The chain loads the ESD for us
  // You must provide the address where it can be found
  fEvent = aESD;
}
//___________________
// void AliFemtoEventReaderESDChain::SetESDfriendSource(AliESDfriend *aFriend)
// {
//   // We need the ESD tree to obtain
//   // information about the friend file location
//   fEventFriend = aFriend;
// }

//____________________________________________________________________
Float_t AliFemtoEventReaderESDChain::GetSigmaToVertex(double *impact, double *covar)
{
  // Calculates the number of sigma to the vertex.

  Float_t b[2];
  Float_t bRes[2];
  Float_t bCov[3];

  b[0] = impact[0];
  b[1] = impact[1];
  bCov[0] = covar[0];
  bCov[1] = covar[1];
  bCov[2] = covar[2];

  bRes[0] = TMath::Sqrt(bCov[0]);
  bRes[1] = TMath::Sqrt(bCov[2]);

  // -----------------------------------
  // How to get to a n-sigma cut?
  //
  // The accumulated statistics from 0 to d is
  //
  // ->  Erf(d/Sqrt(2)) for a 1-dim gauss (d = n_sigma)
  // ->  1 - Exp(-d**2) for a 2-dim gauss (d*d = dx*dx + dy*dy != n_sigma)
  //
  // It means that for a 2-dim gauss: n_sigma(d) = Sqrt(2)*ErfInv(1 - Exp((-x**2)/2)
  // Can this be expressed in a different way?

  if (bRes[0] == 0 || bRes[1] ==0)
    return -1;

  Float_t d = TMath::Sqrt(TMath::Power(b[0]/bRes[0],2) + TMath::Power(b[1]/bRes[1],2));

  // stupid rounding problem screws up everything:
  // if d is too big, TMath::Exp(...) gets 0, and TMath::ErfInverse(1) that should be infinite, gets 0 :(
  if (TMath::Exp(-d * d / 2) < 1e-10)
    return 1000;

  d = TMath::ErfInverse(1 - TMath::Exp(-d * d / 2)) * TMath::Sqrt(2);
  return d;
}




void AliFemtoEventReaderESDChain::CopyESDtoFemtoV0(AliESDv0 *tESDv0, AliFemtoV0 *tFemtoV0, AliESDEvent *tESDevent)
{
  AliESDtrack *pTrack = tESDevent->GetTrack(tESDv0->GetPindex());
  AliESDtrack *nTrack = tESDevent->GetTrack(tESDv0->GetNindex());

  //tFemtoV0->SetdecayLengthV0(tESDv0->DecayLengthV0()); //wrocic do tego
  //tFemtoV0->SetdecayVertexV0X(tESDv0->DecayVertexV0X());
  //tFemtoV0->SetdecayVertexV0Y(tESDv0->DecayVertexV0Y());
  //tFemtoV0->SetdecayVertexV0Z(tESDv0->DecayVertexV0Z()); //nie ma w AliESDv0
  //AliFemtoThreeVector decayvertex(tESDv0->DecayVertexV0X(),tESDv0->DecayVertexV0Y(),tESDv0->DecayVertexV0Z());
  //tFemtoV0->SetdecayVertexV0(decayvertex);
  tFemtoV0->SetdcaV0Daughters(tESDv0->GetDcaV0Daughters());
  double fPrimaryVtxPosition[3];
  tESDevent->GetPrimaryVertex()->GetXYZ(fPrimaryVtxPosition);
  tFemtoV0->SetdcaV0ToPrimVertex(tESDv0->GetD(fPrimaryVtxPosition[0],fPrimaryVtxPosition[1],fPrimaryVtxPosition[2]));

  //jest cos takiego w AliFemtoV0.h czego nie ma w AliAODv0.h
  //void SettpcHitsPos(const int& i);      
  //void SettpcHitsNeg(const int& i);      

  //void SetTrackTopologyMapPos(unsigned int word, const unsigned long& m);
  //void SetTrackTopologyMapNeg(unsigned int word, const unsigned long& m);

  tFemtoV0->SetmomV0X(tESDv0->Px());
  tFemtoV0->SetmomV0Y(tESDv0->Py());
  tFemtoV0->SetmomV0Z(tESDv0->Pz());
  AliFemtoThreeVector momv0(tESDv0->Px(),tESDv0->Py(),tESDv0->Pz());
  tFemtoV0->SetmomV0(momv0);
  tFemtoV0->SetalphaV0(tESDv0->AlphaV0());
  tFemtoV0->SetptArmV0(tESDv0->PtArmV0());
  //tFemtoV0->SeteLambda(tESDv0->ELambda());
  //tFemtoV0->SeteK0Short(tESDv0->EK0Short());
  //tFemtoV0->SetePosProton(tESDv0->EPosProton());
  //tFemtoV0->SeteNegProton(tESDv0->ENegProton());
  tFemtoV0->SetmassLambda(tESDv0->GetEffMass(4,2));
  tFemtoV0->SetmassAntiLambda(tESDv0->GetEffMass(2,4));
  tFemtoV0->SetmassK0Short(tESDv0->GetEffMass(2,2));
  //tFemtoV0->SetrapLambda(tESDv0->RapLambda());
  //tFemtoV0->SetrapK0Short(tESDv0->RapK0Short());
  
  //void SetcTauLambda( float x);   
  //void SetcTauK0Short( float x); 
  
  tFemtoV0->SetptV0(tESDv0->Pt());
  tFemtoV0->SetptotV0(tESDv0->P());
  
  tFemtoV0->SetEtaV0(tESDv0->Eta());
  tFemtoV0->SetPhiV0(tESDv0->Phi());
  tFemtoV0->SetCosPointingAngle(tESDv0->GetV0CosineOfPointingAngle(fPrimaryVtxPosition[0],fPrimaryVtxPosition[1], fPrimaryVtxPosition[2]));
  //tFemtoV0->SetYV0(tESDv0->Y());

  //void SetdedxNeg(float x);
  //void SeterrdedxNeg(float x);//Gael 04Fev2002
  //void SetlendedxNeg(float x);//Gael 04Fev2002
  //void SetdedxPos(float x);
  //void SeterrdedxPos(float x);//Gael 04Fev2002
  //void SetlendedxPos(float x);//Gael 04Fev2002

  //AliAODTrack *trackpos = (AliAODTrack*)tESDv0->GetDaughter(0);
  //AliAODTrack *trackneg = (AliAODTrack*)tESDv0->GetDaughter(1);

  if(pTrack && nTrack)
    {
      tFemtoV0->SetdcaPosToPrimVertex(TMath::Abs(pTrack->GetD(fPrimaryVtxPosition[0],fPrimaryVtxPosition[1],tESDevent->GetMagneticField())));
      tFemtoV0->SetdcaNegToPrimVertex(TMath::Abs(pTrack->GetD(fPrimaryVtxPosition[0],fPrimaryVtxPosition[1],tESDevent->GetMagneticField())));
      double MomPos[3];
      pTrack->PxPyPz(MomPos);
      tFemtoV0->SetmomPosX(MomPos[0]);
      tFemtoV0->SetmomPosY(MomPos[1]);
      tFemtoV0->SetmomPosZ(MomPos[2]);
      AliFemtoThreeVector mompos(MomPos[0],MomPos[1],MomPos[2]);
      tFemtoV0->SetmomPos(mompos);

      double MomNeg[3];
      nTrack->PxPyPz(MomNeg);
      tFemtoV0->SetmomNegX(MomNeg[0]);
      tFemtoV0->SetmomNegY(MomNeg[1]);
      tFemtoV0->SetmomNegZ(MomNeg[2]);
      AliFemtoThreeVector momneg(MomNeg[0],MomNeg[1],MomNeg[2]);
      tFemtoV0->SetmomNeg(momneg);

      tFemtoV0->SetptPos(pTrack->Pt());
      tFemtoV0->SetptotPos(pTrack->P());
      tFemtoV0->SetptNeg(nTrack->Pt());
      tFemtoV0->SetptotNeg(nTrack->P());
      
      tFemtoV0->SetidNeg(nTrack->GetID());
      //cout<<"tESDv0->GetNegID(): "<<tESDv0->GetNegID()<<endl;
      //cout<<"tFemtoV0->IdNeg(): "<<tFemtoV0->IdNeg()<<endl;
      tFemtoV0->SetidPos(pTrack->GetID());
      
      tFemtoV0->SetEtaPos(pTrack->Eta());
      tFemtoV0->SetEtaNeg(nTrack->Eta());

      tFemtoV0->SetEtaPos(pTrack->Eta()); //tESDv0->PseudoRapPos()
      tFemtoV0->SetEtaNeg(nTrack->Eta()); //tESDv0->PseudoRapNeg()
      tFemtoV0->SetTPCNclsPos(pTrack->GetTPCNcls());
      tFemtoV0->SetTPCNclsNeg(nTrack->GetTPCNcls());
      tFemtoV0->SetTPCclustersPos(pTrack->GetTPCClusterMap());
      tFemtoV0->SetTPCclustersNeg(nTrack->GetTPCClusterMap());
      tFemtoV0->SetTPCsharingPos(pTrack->GetTPCSharedMap());
      tFemtoV0->SetTPCsharingNeg(nTrack->GetTPCSharedMap());
      tFemtoV0->SetNdofPos(pTrack->GetTPCchi2()/pTrack->GetTPCNcls());
      tFemtoV0->SetNdofNeg(nTrack->GetTPCchi2()/nTrack->GetTPCNcls());
      tFemtoV0->SetStatusPos(pTrack->GetStatus());
      tFemtoV0->SetStatusNeg(nTrack->GetStatus());

      double fV1[3];
      fEvent->GetPrimaryVertexTPC()->GetXYZ(fV1);
      double xtpc[3]; AliFemtoThreeVector tmpVec;
      pTrack->GetInnerXYZ(xtpc); xtpc[2] -= fV1[2];
      tmpVec.SetX(xtpc[0]); tmpVec.SetY(xtpc[1]); tmpVec.SetZ(xtpc[2]);
      tFemtoV0->SetNominalTpcEntrancePointPos(tmpVec);
      nTrack->GetInnerXYZ(xtpc); xtpc[2] -= fV1[2];
      tmpVec.SetX(xtpc[0]); tmpVec.SetY(xtpc[1]); tmpVec.SetZ(xtpc[2]);
      tFemtoV0->SetNominalTpcEntrancePointNeg(tmpVec);

      pTrack->GetOuterXYZ(xtpc); xtpc[2] -= fV1[2];
      tmpVec.SetX(xtpc[0]); tmpVec.SetY(xtpc[1]); tmpVec.SetZ(xtpc[2]);
      tFemtoV0->SetNominalTpcExitPointPos(tmpVec);
      nTrack->GetOuterXYZ(xtpc); xtpc[2] -= fV1[2];
      tmpVec.SetX(xtpc[0]); tmpVec.SetY(xtpc[1]); tmpVec.SetZ(xtpc[2]);
      tFemtoV0->SetNominalTpcExitPointNeg(tmpVec);


      if (fESDpid) {
	tFemtoV0->SetPosNSigmaTPCK(fESDpid->NumberOfSigmasTPC(pTrack,AliPID::kKaon));
	tFemtoV0->SetNegNSigmaTPCK(fESDpid->NumberOfSigmasTPC(nTrack,AliPID::kKaon));
	tFemtoV0->SetPosNSigmaTPCP(fESDpid->NumberOfSigmasTPC(pTrack,AliPID::kProton));
	tFemtoV0->SetNegNSigmaTPCP(fESDpid->NumberOfSigmasTPC(nTrack,AliPID::kProton));
	tFemtoV0->SetPosNSigmaTPCPi(fESDpid->NumberOfSigmasTPC(pTrack,AliPID::kPion));
	tFemtoV0->SetNegNSigmaTPCPi(fESDpid->NumberOfSigmasTPC(nTrack,AliPID::kPion));
      }
      else {
	tFemtoV0->SetPosNSigmaTPCK(-9999);
	tFemtoV0->SetNegNSigmaTPCK(-9999);
	tFemtoV0->SetPosNSigmaTPCP(-9999);
	tFemtoV0->SetNegNSigmaTPCP(-9999);
	tFemtoV0->SetPosNSigmaTPCPi(-9999);
	tFemtoV0->SetNegNSigmaTPCPi(-9999);
      }

      if((tFemtoV0->StatusPos()&AliESDtrack::kTOFpid)==0 || (tFemtoV0->StatusPos()&AliESDtrack::kTIME)==0 || (tFemtoV0->StatusPos()&AliESDtrack::kTOFout)==0 || (tFemtoV0->StatusPos()&AliESDtrack::kTOFmismatch)>0)
	{
	  if((tFemtoV0->StatusNeg()&AliESDtrack::kTOFpid)==0 || (tFemtoV0->StatusNeg()&AliESDtrack::kTIME)==0 || (tFemtoV0->StatusNeg()&AliESDtrack::kTOFout)==0 || (tFemtoV0->StatusNeg()&AliESDtrack::kTOFmismatch)>0)
	    {
	      tFemtoV0->SetPosNSigmaTOFK(-9999);
	      tFemtoV0->SetNegNSigmaTOFK(-9999);
	      tFemtoV0->SetPosNSigmaTOFP(-9999);
	      tFemtoV0->SetNegNSigmaTOFP(-9999);
	      tFemtoV0->SetPosNSigmaTOFPi(-9999);
	      tFemtoV0->SetNegNSigmaTOFPi(-9999);
	    }
	}
      else
	{
	  if (fESDpid) {
	    tFemtoV0->SetPosNSigmaTOFK(fESDpid->NumberOfSigmasTOF(pTrack,AliPID::kKaon));
	    tFemtoV0->SetNegNSigmaTOFK(fESDpid->NumberOfSigmasTOF(nTrack,AliPID::kKaon));
	    tFemtoV0->SetPosNSigmaTOFP(fESDpid->NumberOfSigmasTOF(pTrack,AliPID::kProton));
	    tFemtoV0->SetNegNSigmaTOFP(fESDpid->NumberOfSigmasTOF(nTrack,AliPID::kProton));
	    tFemtoV0->SetPosNSigmaTOFPi(fESDpid->NumberOfSigmasTOF(pTrack,AliPID::kPion));
	    tFemtoV0->SetNegNSigmaTOFPi(fESDpid->NumberOfSigmasTOF(nTrack,AliPID::kPion));
	  }
	  else {
	    tFemtoV0->SetPosNSigmaTOFK(-9999);
	    tFemtoV0->SetNegNSigmaTOFK(-9999);
	    tFemtoV0->SetPosNSigmaTOFP(-9999);
	    tFemtoV0->SetNegNSigmaTOFP(-9999);
	    tFemtoV0->SetPosNSigmaTOFPi(-9999);
	    tFemtoV0->SetNegNSigmaTOFPi(-9999);
	  }
	}
    }
  else
    {
      tFemtoV0->SetStatusPos(999);
      tFemtoV0->SetStatusNeg(999);
    }
  tFemtoV0->SetOnFlyStatusV0(tESDv0->GetOnFlyStatus());
}




void AliFemtoEventReaderESDChain::SetReadTrackType(ReadTrackType aType)
{
  fTrackType = aType;
}

//trigger
void AliFemtoEventReaderESDChain::SetEventTrigger(UInt_t eventtrig)
{
  fEventTrig = eventtrig;
}

//V0 reading
void AliFemtoEventReaderESDChain::SetReadV0(bool a)
{
  fReadV0 = a;
}





